@@ -1,4 +1,4 @@
-# $Id: mvt.R 257 2013-09-16 07:45:39Z thothorn $
+# $Id: mvt.R 275 2014-05-02 22:07:27Z mmaechler $
 
 chkcorr <- function(x) {
 
@@ -128,9 +128,8 @@ pmvnorm <- function(lower=-Inf, upper=Inf, mean=rep(0, length(lower)), corr=NULL
                      algorithm = algorithm, ...)
       }
     }
-    attr(RET$value, "error") <- RET$error
-    attr(RET$value, "msg") <- RET$msg
-    return(RET$value)
+    ## return
+    structure(RET$value, "error" = RET$error, "msg" = RET$msg)
 }
 
 pmvt <- function(lower=-Inf, upper=Inf, delta=rep(0, length(lower)),
@@ -224,13 +223,13 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
     infin[isNInf(lower) & isInf(upper)] <- -1
 
     ### fix for Miwa algo:
-    ### pmvnorm(lower=c(-Inf, 0, 0), upper=c(0, Inf, Inf), 
-    ###         mean=c(0, 0, 0), sigma=S, algorithm = Miwa()) 
+    ### pmvnorm(lower=c(-Inf, 0, 0), upper=c(0, Inf, Inf),
+    ###         mean=c(0, 0, 0), sigma=S, algorithm = Miwa())
     ###         returned NA
 
     if (class(algorithm) == "Miwa") {
         if (any(infin == -1) & n >= 3) {
-            WhereBothInfIs <- which(infin == -1) 
+            WhereBothInfIs <- which(infin == -1)
             n <- n - length(WhereBothInfIs)
             corr <- corr[-WhereBothInfIs, -WhereBothInfIs]
             upper <- upper[-WhereBothInfIs]
@@ -240,7 +239,7 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
         if (any(infin == 0) & n >= 2) {
             WhereNegativInfIs <- which(infin==0)
             inversecorr <- rep(1, n)
-            inversecorr[WhereNegativInfIs] <- -1   
+            inversecorr[WhereNegativInfIs] <- -1
             corr <- diag(inversecorr) %*% corr %*% diag(inversecorr)
             infin[WhereNegativInfIs] <- 1
 
@@ -248,7 +247,7 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
             upper[WhereNegativInfIs] <- -lower[WhereNegativInfIs]
             lower[WhereNegativInfIs] <- -tempsaveupper
         }
-    } 
+    }
 
     ### this is a bug in `mvtdst' not yet fixed
     if (all(infin < 0))
@@ -297,47 +296,42 @@ rmvt <- function(n, sigma = diag(2), df = 1,
            stop("wrong 'type'"))
 }
 
-dmvt <- function(x, delta, sigma, df = 1,
+dmvt <- function(x, delta = rep(0, p), sigma = diag(p), df = 1,
                  log = TRUE, type = "shifted")
 {
+    if (is.vector(x))
+        x <- matrix(x, ncol = length(x))
+    p <- ncol(x)
     if (df == 0 || isInf(df)) # MH: now (also) properly allow df = Inf
         return(dmvnorm(x, mean = delta, sigma = sigma, log = log))
-
+    if(!missing(delta)) {
+	if(!is.null(dim(delta))) dim(delta) <- NULL
+	if (length(delta) != p)
+	    stop("delta and sigma have non-conforming size")
+    }
+    if(!missing(sigma)) {
+	if (p != ncol(sigma))
+	    stop("x and sigma have non-conforming size")
+	if (!isSymmetric(sigma, tol = sqrt(.Machine$double.eps),
+			 check.attributes = FALSE))
+	    stop("sigma must be a symmetric matrix")
+    }
     type <- match.arg(type)
 
-    if (is.vector(x)) {
-        x <- matrix(x, ncol = length(x))
+    dec <- tryCatch(chol(sigma), error=function(e)e)
+    if (inherits(dec, "error")) {
+	x.is.d <- colSums(t(x) != delta) == 0
+	logretval <- rep.int(-Inf, nrow(x))
+	logretval[x.is.d] <- Inf # and all other f(.) == 0
+    } else {
+	R.x_m <- backsolve(dec, t(x) - delta, transpose = TRUE)
+	rss <- colSums(R.x_m ^ 2)
+	logretval <- lgamma((p + df)/2) -
+	    (lgamma(df / 2) + sum(log(diag(dec))) + p/2 * log(pi * df)) -
+		0.5 * (df + p) * log1p(rss / df)
     }
-    if (missing(delta)) {
-        delta <- rep(0, length = ncol(x))
-    }
-    if (missing(sigma)) {
-        sigma <- diag(ncol(x))
-    }
-    if (NCOL(x) != NCOL(sigma)) {
-        stop("x and sigma have non-conforming size")
-    }
-    if (!isSymmetric(sigma, tol = sqrt(.Machine$double.eps),
-                     check.attributes = FALSE)) {
-        stop("sigma must be a symmetric matrix")
-    }
-    if (length(delta) != NROW(sigma)) {
-        stop("mean and sigma have non-conforming size")
-    }
-
-    m <- NCOL(sigma)
-
-    distval <- mahalanobis(x, center = delta, cov = sigma)
-
-    logdet <- sum(log(eigen(sigma, symmetric = TRUE,
-                            only.values = TRUE)$values))
-
-    logretval <- lgamma((m + df)/2) -
-                 (lgamma(df / 2) + 0.5 * (logdet + m * log(pi * df))) -
-                 0.5 * (df + m) * log(1 + distval / df)
-    if (log)
-        return(logretval)
-    return(exp(logretval))
+    names(logretval) <- rownames(x)
+    if (log) logretval else exp(logretval)
 }
 
 ### find suitable interval to search for quantile
@@ -384,8 +378,7 @@ qmvnorm <- function(p, interval = NULL,
     dim <- length(mean)
     if (is.matrix(corr)) dim <- nrow(corr)
     if (is.matrix(sigma)) dim <- nrow(sigma)
-    lower <- rep(0, dim)
-    upper <- rep(0, dim)
+    lower <- upper <- rep.int(0, dim)
     args <- checkmvArgs(lower, upper, mean, corr, sigma)
     if (args$uni) {
         if (is.null(args$sigma))
@@ -393,8 +386,7 @@ qmvnorm <- function(p, interval = NULL,
         if (tail == "both.tails") p <- ifelse(p < 0.5, p / 2, 1 - (1 - p)/2)
         q <- qnorm(p, mean = args$mean, sd = args$sigma,
                    lower.tail = (tail != "upper.tail"))
-        qroot <- list(quantile = q, f.quantile = 0)
-        return(qroot)
+        return( list(quantile = q, f.quantile = 0) )
     }
     dim <- length(args$mean)
 
@@ -421,7 +413,7 @@ qmvnorm <- function(p, interval = NULL,
             interval <- approx_interval(p = p, tail = tail,
                                         corr = cr, df = 0)
         } else {
-            interval <- c(ifelse(tail == "both.tails", 0, -10), 10)
+            interval <- c(if(tail == "both.tails") 0 else -10, 10)
         }
     }
 
@@ -454,8 +446,7 @@ qmvt <- function(p, interval = NULL,
     dim <- 1
     if (!is.null(corr)) dim <- NROW(corr)
     if (!is.null(sigma)) dim <- NROW(sigma)
-    lower <- rep(0, dim)
-    upper <- rep(0, dim)
+    lower <- upper <- rep.int(0, dim)
     args <- checkmvArgs(lower, upper, delta, corr, sigma)
     if (args$uni) {
         if (tail == "both.tails") p <- ifelse(p < 0.5, p / 2, 1 - (1 - p)/2)
@@ -492,29 +483,27 @@ qmvt <- function(p, interval = NULL,
             interval <- approx_interval(p = p, tail = tail,
                                         corr = cr, df = df)
         } else {
-            interval <- c(ifelse(tail == "both.tails", 0, -10), 10)
+            interval <- c(if(tail == "both.tails") 0 else -10, 10)
         }
     }
 
-    if (is.null(dots$uniroot)) {
-        qroot <- uniroot(pfct, interval = interval)
-    } else {
-        qroot <- do.call("uniroot", list(pfct, interval = interval, dots$uniroot))
-    }
+    qroot <-
+        if (is.null(dots$uniroot)) {
+            uniroot(pfct, interval = interval)
+        } else {
+            do.call("uniroot", list(pfct, interval = interval, dots$uniroot))
+        }
     names(qroot)[1:2] <- c("quantile", "f.quantile")
     qroot
 }
 
 GenzBretz <- function(maxpts = 25000, abseps = 0.001, releps = 0) {
-    ret <- list(maxpts = maxpts, abseps = abseps, releps = releps)
-    class(ret) <- "GenzBretz"
-    ret
+    structure(list(maxpts = maxpts, abseps = abseps, releps = releps),
+              class = "GenzBretz")
 }
 
 Miwa <- function(steps = 128) {
-    ret <- list(steps = steps)
-    class(ret) <- "Miwa"
-    ret
+    structure(list(steps = steps), class = "Miwa")
 }
 
 probval <- function(x, ...)
@@ -561,8 +550,7 @@ probval.Miwa <- function(x, n, df, lower, upper, infin, corr, corrF, delta) {
                          upper = as.double(upper),
                          lower = as.double(lower),
                          infin = as.integer(infin))
-    ret <- list(value = p, inform = 0, error = NA)
-    ret
+    list(value = p, inform = 0, error = NA)
 }
 
 dots2GenzBretz <- function(...) {
